package aventi.epark.activities;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;

import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.toolbox.Volley;
import com.leocardz.aelv.library.Aelv;
import com.leocardz.aelv.library.AelvCustomAction;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import aventi.epark.R;
import aventi.epark.phprequests.HistoryRequest;
import aventi.epark.utilities.ListAdapter;
import aventi.epark.utilities.ListItem;

public class HistoryActivity extends Activity {
    // Initializing local variables
    private String UID;
    private ArrayList<ListItem> listItems;
    private String[][] mStringArray;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_my_account);

        // Get data from MainActivity
        Intent intent = getIntent();
        UID = intent.getStringExtra("user_id");

        // ResponseListener handling history-requests that are put in the queue
        Response.Listener<JSONArray> responseListener = new Response.Listener<JSONArray>() {
            @Override
            public void onResponse(JSONArray response) {
                try{
                    JSONArray jsonArray = response;

                    // Initializing mStringArray with correct dimensions
                    mStringArray = new String[3][jsonArray.length()];

                    for(int i = 0; i < jsonArray.length(); i++){
                        JSONObject jsonObject = jsonArray.getJSONObject(i);
                        mStringArray[0][i] = jsonObject.getString("timeFrom");
                        mStringArray[1][i] = jsonObject.getString("timeTo");
                        mStringArray[2][i] = jsonObject.getString("beacon_id");
                    }
                    historyItems();
                }catch (JSONException e){
                    e.printStackTrace();
                }
            }
        };
        HistoryRequest historyRequest = null;
        try {
            historyRequest = new HistoryRequest(UID, responseListener);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        RequestQueue queue = Volley.newRequestQueue(HistoryActivity.this);
        queue.add(historyRequest);
    }

    // Creating a ListView with ListItems from server
    public void historyItems() {
        final int COLLAPSED_HEIGHT_1 = 150, COLLAPSED_HEIGHT_2 = 200, COLLAPSED_HEIGHT_3 = 250;
        final int EXPANDED_HEIGHT_1 = 250, EXPANDED_HEIGHT_2 = 300, EXPANDED_HEIGHT_3 = 350, EXPANDED_HEIGHT_4 = 400;

        ListView listView = (ListView) findViewById(R.id.list);

        listItems = new ArrayList<ListItem>();

        ListAdapter adapter = new ListAdapter(this, R.layout.list_item, listItems);

        listView.setAdapter(adapter);

        // Setup
        final Aelv aelv = new Aelv(true, 200, listItems, listView, adapter, new AelvCustomAction() {
            @Override
            public void onEndAnimation(int position) {
                listItems.get(position).setDrawable(listItems.get(position).isOpen() ? R.mipmap.up : R.mipmap.down);
            }
        });

        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                aelv.toggle(view, position);
            }
        });

        // Initalizing ListItem objects to fill the ListView
        for(int i = 0; i < mStringArray[0].length; i++) {
            ListItem listItem = new ListItem(getDate(mStringArray[0][i]) + "\n" + "\n" + getDuration(mStringArray[0][i],mStringArray[1][i]));
            listItem.setUp(COLLAPSED_HEIGHT_2, EXPANDED_HEIGHT_4, false);
            listItems.add(listItem);
        }
    }

    public String getDate(String timeStamp) {
        String date;

        String day = Character.toString(timeStamp.charAt(8))+Character.toString(timeStamp.charAt(9));
        String month = Character.toString(timeStamp.charAt(5))+Character.toString(timeStamp.charAt(6));
        String year = Character.toString(timeStamp.charAt(0))+Character.toString(timeStamp.charAt(1))+Character.toString(timeStamp.charAt(2))+Character.toString(timeStamp.charAt(3));

        date = day + "/" + month + " - " + year;

        return date;
    }

    public String getDuration(String from, String to) {
        String duration = null;

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

        Date t1 = null;
        Date t2 = null;

        try {
            t1 = format.parse(from);
            t2 = format.parse(to);

            // Time difference in milliseconds
            long diff = t2.getTime() - t1.getTime();

            long diffMinutes = diff / (60 * 1000) % 60;
            long diffHours = diff / (60 * 60 * 1000) % 24;
            long diffDays = diff / (24 * 60 * 60 * 1000);

            if(diffDays > 0){
                duration = diffDays + "d " + diffHours + "h " + diffMinutes + "m";
            } else if(diffHours > 0) {
                duration = diffHours + "h " + diffMinutes + "m";
            } else{
                duration = diffMinutes + "m";
            }

        } catch (ParseException e) {
            e.printStackTrace();
        }

        return duration;
    }
}

