package aventi.epark.activities;

import android.content.Intent;
import android.os.RemoteException;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.util.ArrayMap;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import org.altbeacon.beacon.Beacon;
import org.altbeacon.beacon.BeaconConsumer;
import org.altbeacon.beacon.BeaconManager;
import org.altbeacon.beacon.BeaconParser;
import org.altbeacon.beacon.Identifier;
import org.altbeacon.beacon.RangeNotifier;
import org.altbeacon.beacon.Region;

import java.util.Collection;
import java.util.Map;

import aventi.epark.R;

public class MainActivity extends AppCompatActivity implements View.OnClickListener, BeaconConsumer, RangeNotifier {

    Button btnPark, btnMap;
    String UID;

    private BeaconManager mBeaconManager;
    private Map<Identifier, Double> availableSpots = new ArrayMap<>();


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Toolbar mToolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);

        final EditText etFirstName = (EditText) findViewById(R.id.etFirstName);
        final EditText etLastName = (EditText) findViewById(R.id.etLastName);
        final EditText etUsername = (EditText) findViewById(R.id.etUsername);
        final TextView welcomeMessage = (TextView) findViewById(R.id.tvWelcomeMsg);

        Intent intent = getIntent();
        String LastName = intent.getStringExtra("LastName");
        String FirstName = intent.getStringExtra("FirstName");
        String email = intent.getStringExtra("email");
        UID = intent.getStringExtra("user_id");

        String message = FirstName + " " + LastName;
        welcomeMessage.setText(message);

        btnPark = (Button) findViewById(R.id.btnPark);
        btnPark.setOnClickListener(this);

        btnMap = (Button) findViewById(R.id.btnMap);
        btnMap.setOnClickListener(this);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnPark:
                //Toast.makeText(this, "Park button pressed" + availableSpots, Toast.LENGTH_SHORT).show();
                for (Map.Entry entry : availableSpots.entrySet()) {
                    Log.i("parkingtest", "instanceID: " + entry.getKey() + " distance: " + entry.getValue());
                }
                Identifier check = findNearestBeacon(availableSpots);
                Log.i("parkingtest", "Chosen parking slot: " + check);
                if (check != null){
                    Log.i("parkingtest", "The ID of the chosen parking is: " + identifierToInt(check));
                    Toast.makeText(this, "Parking ID: " + identifierToInt(check), Toast.LENGTH_SHORT).show();
                    Intent intent = new Intent(getBaseContext(),TicketActivity.class);
                    intent.putExtra("chosenParking", String.valueOf(identifierToInt(check)));
                    intent.putExtra("user_id", UID);
                    startActivity(intent);
                }
                break;

            case R.id.btnMap:
                Intent intent = new Intent(getBaseContext(),MapActivity.class);
                startActivity(intent);
                break;
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater mMenuInflater = getMenuInflater();
        mMenuInflater.inflate(R.menu.epark_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_setting:
                startActivity(new Intent(this, SettingsActivity.class));
                break;
            case R.id.action_about:
                // Display something about the app
                break;
            case R.id.menu_my_account:
                Intent intent = new Intent(getBaseContext(),HistoryActivity.class);
                intent.putExtra("user_id", UID);
                startActivity(intent);
                break;
            case R.id.menu_log_out:
                startActivity(new Intent(this, LoginActivity.class));
                finish();
                break;
        }
        return super.onOptionsItemSelected(item);
    }
//------------------------------------------------------------------------
//-------------------------   BEACON SERVICE   ---------------------------
//------------------------------------------------------------------------
    /**---------------------------------
     * Fascilitate commands from BeaconManager
     * ---------------------------------
     * This method initializes functions from the Beacon-library,
     * when the service is up and running*/
    @Override
    public void onBeaconServiceConnect() {
        Region region = new Region("all-beacons-region", null, null, null);
        try {
            mBeaconManager.startRangingBeaconsInRegion(region);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        mBeaconManager.setRangeNotifier(this);
    }

    /**---------------------------------
     * Scan and range beacons in a defined region
     * ---------------------------------
     * This method maps scan-results; namespaceID, instanceID and distance in meters*/
    @Override
    public void didRangeBeaconsInRegion(Collection<Beacon> collection, Region region) {
        if (availableSpots != null && availableSpots.size() != 0) {
            availableSpots.clear(); // Refresh map by clearing
        }
        for (Beacon beacon : collection) {
            Log.i("Beacon Services", "Checking if UID matches Eddystone Standard");
            if (beacon.getServiceUuid() == 0xfeaa && beacon.getBeaconTypeCode() == 0x00) {
                // Extracting Eddystone UID ID-frames
                Identifier namespaceID = beacon.getId1();
                Identifier instanceID = beacon.getId2();
                double distance = beacon.getDistance();
                Log.i("Beacon Services", "AVAILABLE BEACON [namespace id: " + namespaceID +
                        " , instance id: " + instanceID + " , distance: " + distance + " meters]");
                availableSpots.put(instanceID, distance);
            }
        }

    }

    @Override
    protected void onResume() {
        super.onResume();
        mBeaconManager = BeaconManager.getInstanceForApplication(this.getApplicationContext());
        // Detect the main Eddystone UID frame:
        mBeaconManager.getBeaconParsers().add(new BeaconParser().setBeaconLayout("s:0-1=feaa,m:2-2=00,p:3-3:-41,i:4-13,i:14-19"));
        mBeaconManager.bind(this);
    }

    @Override
    protected void onPause() {
        super.onPause();
        mBeaconManager.unbind(this);
    }

    /**---------------------------------------------
     * Find the closest Beacon in meters.
     * ---------------------------------------------
     * This method puts all beacons the Bluetooth service
     * gets signals from and puts them in a updating map.
     * The method returns the closest Beacon at the given time
     * ---------------------------------------------*/
    public Identifier findNearestBeacon(Map<Identifier, Double> beacons) {
        Double minimumDistance = 100.0;
        Identifier nearestBeacon = null;
        for (Map.Entry entry : beacons.entrySet()) {
            if ((Double) entry.getValue() < minimumDistance) {
                minimumDistance = (Double) entry.getValue();
                nearestBeacon = (Identifier) entry.getKey();
            }
        }
        return nearestBeacon;
    }
    /**---------------------------------------------
     * Convert the Java type Identifier to type int.
     * ---------------------------------------------
     * This method takes in an Identifier, and converts it to a string.
     * The string represents the beacon ID as a hexadecimal number,
     * and is converted to an int ranging from 0-65535.
     * ---------------------------------------------*/
    public int identifierToInt(Identifier id) {
        int sum = 0;
        String str = id.toString();
        int index = 13;
        int p = 0;
        while (index > 1 && str.charAt(index) != '0'){
            int pow = (int) Math.pow(16.0,(double) p);
            if (str.charAt(index) > '0' && str.charAt(index) <= '9'){
                sum += (((int)str.charAt(index))-48)*pow;
            }else{
                sum += (((int)str.charAt(index))-87)*pow;
            }
            index--;
            p++;
        }
        return sum;
    }
}
