package aventi.epark.activities;

import android.os.RemoteException;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.widget.ImageView;
import android.widget.TextView;

import org.altbeacon.beacon.Beacon;
import org.altbeacon.beacon.BeaconConsumer;
import org.altbeacon.beacon.BeaconManager;
import org.altbeacon.beacon.BeaconParser;
import org.altbeacon.beacon.Identifier;
import org.altbeacon.beacon.RangeNotifier;
import org.altbeacon.beacon.Region;
import org.apache.commons.math3.fitting.leastsquares.LeastSquaresOptimizer;
import org.apache.commons.math3.fitting.leastsquares.LevenbergMarquardtOptimizer;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collection;
import java.util.List;

import aventi.epark.R;
import aventi.epark.navigation.CellIdentification;
import aventi.epark.navigation.Multilateration;
import aventi.epark.navigation.trilateration.NonLinearLeastSquaresSolver;
import aventi.epark.navigation.trilateration.TrilaterationFunction;

public class MapActivity extends AppCompatActivity implements BeaconConsumer, RangeNotifier {

    private int accuracyTesting = 1; // Set to 1 to output testmeasurements, 0 to ignore

    private BeaconManager beaconManager;
    private ImageView ivDot;
    private ImageView testmap;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.test_map);


        ivDot = (ImageView) findViewById(R.id.imageView4);
        testmap = (ImageView) findViewById(R.id.imageView3);
    }

    //------------------------------------------------------------------------
//-------------------------   BEACON SERVICE   ---------------------------
//------------------------------------------------------------------------
    /**---------------------------------
     * Fascilitate commands from BeaconManager
     * ---------------------------------
     * This method initializes functions from the Beacon-library,
     * when the service is up and running*/
    @Override
    public void onBeaconServiceConnect() {
        Region region = new Region("all-beacons-region", null, null, null);
        try {
            beaconManager.startRangingBeaconsInRegion(region);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        beaconManager.setRangeNotifier(this);
    }

    @Override
    public void didRangeBeaconsInRegion(Collection<Beacon> collection, Region region) {
        for (Beacon beacon : collection) {
            Log.i("Beacon Services", "Checking if UID matches Eddystone Standard");
            if (beacon.getServiceUuid() == 0xfeaa && beacon.getBeaconTypeCode() == 0x00) {
                if( accuracyTesting == 1 ){
                    Log.i("SingleBeaconRangeTest", String.valueOf(beacon.getRssi()));
                }
            }
        }
        if (collection.size() > 3 || accuracyTesting != 0) {
            double maxCoordX = 10.0;
            double maxCoordY = 10.0;

            int index = 0;

            switch (index){
                case 0:
                    Multilateration multilateration = new Multilateration(collection);
                    double[] pos = multilateration.getCurrentLocation();
                    ivDot.setX((float) (pos[0]/maxCoordX)*testmap.getWidth());
                    ivDot.setY((float) (pos[0]/maxCoordY)*testmap.getHeight());

                    Log.i("PositionService: ", String.valueOf(pos[0]) + ", " + String.valueOf(pos[1]) );
                    break;
                case 1:
                    CellIdentification cellIdentification = new CellIdentification(collection);
                    double[] posCell = cellIdentification.getCurrentLocation(collection);
                    ivDot.setX((float) (posCell[0]/maxCoordX)*testmap.getWidth());
                    ivDot.setY((float) (posCell[0]/maxCoordY)*testmap.getHeight());
                    break;
                case 2:
                    /**TEST SYSTEM - FIXED INPUT
                     * System used to test the positioning algorithm with
                     * fixed input.*/
                    double[][] posTest = new double[][] {{0,0},{10,0},{0,10},{10,10}};
                    double[] distanceTest = new double[] {7.1, 7.1, 7.1, 7.1};

                    NonLinearLeastSquaresSolver solver = new NonLinearLeastSquaresSolver(new TrilaterationFunction(posTest,distanceTest),new LevenbergMarquardtOptimizer());
                    LeastSquaresOptimizer.Optimum optimum = solver.solve();
                    double[] myPos = optimum.getPoint().toArray();
                    Log.i("Position: ", myPos[0] + " , " + myPos[1]);
                    break;
            }
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        beaconManager = BeaconManager.getInstanceForApplication(this.getApplicationContext());
        // Detect the main Eddystone UID frame:
        beaconManager.getBeaconParsers().add(new BeaconParser().setBeaconLayout("s:0-1=feaa,m:2-2=00,p:3-3:-41,i:4-13,i:14-19"));
        beaconManager.bind(this);
    }

    @Override
    protected void onPause() {
        super.onPause();
        beaconManager.unbind(this);
    }
}
