package aventi.epark.activities;

import android.content.DialogInterface;
import android.content.Intent;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.toolbox.Volley;

import org.json.JSONException;
import org.json.JSONObject;

import java.math.BigDecimal;
import java.math.RoundingMode;

import aventi.epark.R;
import aventi.epark.phprequests.CompleteRequest;
import aventi.epark.phprequests.TicketRequest;
import aventi.epark.phprequests.TimingRequest;
import aventi.epark.utilities.Stopwatch;

public class TicketActivity extends AppCompatActivity implements View.OnClickListener {
    private int queryFlag;
    private String timeFrom;
    private TextView tvParkingTimer;
    private TextView tvParkingCost;
    private  String chosenParking;
    private String UID;

    Stopwatch parkingTimer = new Stopwatch();

    android.os.Handler customHandler = new android.os.Handler();

    @Override
    protected void onResume() {
        super.onResume();
        queryFlag = 0;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_ticket);

        Intent intent = getIntent();
        chosenParking = intent.getStringExtra("chosenParking");
        final String LastName = intent.getStringExtra("LastName");
        final String FirstName = intent.getStringExtra("FirstName");
        UID = intent.getStringExtra("user_id");
        final TextView tvChosenParking = (TextView) findViewById(R.id.tvChosenParking);
        tvParkingTimer = (TextView) findViewById(R.id.tvParkingTimer);
        tvParkingCost = (TextView) findViewById(R.id.tvParkingCost);

        final Button btnComplete = (Button) findViewById(R.id.btnCompleteParking);
        btnComplete.setOnClickListener(this);

        customHandler.postDelayed(updateParkingTimer, 0);


        Response.Listener<String> responseListener = new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
                try {
                    JSONObject jsonResponse = new JSONObject(response);
                    boolean success = jsonResponse.getBoolean("success");

                    if (success) {
                        if (queryFlag == 1) {
                            timeFrom = jsonResponse.getString("timeFrom");
                            String ticketHeader = "Parked at slot: " + chosenParking + "\n" + "From: " + timeFrom;
                            tvChosenParking.setText(ticketHeader);
                            parkingTimer.start();
                        }
                        queryFlag++;
                    } else {
                        AlertDialog.Builder builder = new AlertDialog.Builder(TicketActivity.this);
                        builder.setMessage("Error")
                                .setNegativeButton("Retry", null)
                                .create()
                                .show();
                    }
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        };


        TicketRequest ticketRequest = new TicketRequest(UID, chosenParking, responseListener);
        TimingRequest timingRequest = new TimingRequest(UID, chosenParking, responseListener);
        RequestQueue queue = Volley.newRequestQueue(TicketActivity.this);
        queue.add(ticketRequest);
        queue.add(timingRequest);
    }

    private Runnable updateParkingTimer = new Runnable() {
        @Override
        public void run() {
            tvParkingTimer.setText(parkingTimer.toString());
            calculateCost(parkingTimer.getElapsedTimeMin());
            customHandler.postDelayed(this, 1000);

        }
    };

    public void calculateCost(long minutes){
        double cost = 0.00;
        cost = ((double)minutes)/60;
        Double truncatedCost = BigDecimal.valueOf(cost).setScale(3, RoundingMode.HALF_UP).doubleValue();
        String strCost = String.format("%s", truncatedCost);
        tvParkingCost.setText("$"+strCost);
    }

    DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
        @Override
        public void onClick(DialogInterface dialogInterface, int i) {
            switch (i) {
                case DialogInterface.BUTTON_POSITIVE:
                    //Yes button clicked
                    Response.Listener<String> responseListener = new Response.Listener<String>() {
                        @Override
                        public void onResponse(String response) {
                            try {
                                JSONObject jsonResponse = new JSONObject(response);
                                boolean success = jsonResponse.getBoolean("success");

                                if (success) {
                                    finish(); // Finishes the activity -> back to main

                                } else {
                                    AlertDialog.Builder builder = new AlertDialog.Builder(TicketActivity.this);
                                    builder.setMessage("Error")
                                            .setNegativeButton("Retry", null)
                                            .create()
                                            .show();
                                }
                            } catch (JSONException e) {
                                e.printStackTrace();
                            }
                        }
                    };
                    CompleteRequest completeRequest = new CompleteRequest(UID, chosenParking, responseListener);
                    RequestQueue queue = Volley.newRequestQueue(TicketActivity.this);
                    queue.add(completeRequest);
                    break;

                case DialogInterface.BUTTON_NEGATIVE:
                    //No button clicked
                    break;
            }
        }
    };

    @Override
    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.btnCompleteParking:
                AlertDialog.Builder builder = new AlertDialog.Builder(TicketActivity.this);
                builder.setMessage("Complete Parking?").setPositiveButton("Yes", dialogClickListener).setNegativeButton("No", dialogClickListener).show();
                break;
        }
    }
}
