package aventi.epark.navigation;

import android.support.v4.content.res.TypedArrayUtils;
import android.util.ArrayMap;
import android.util.Log;

import org.altbeacon.beacon.Beacon;
import org.altbeacon.beacon.Identifier;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * Created by espeland on 02.03.18.
 */

public class CellIdentification {

    private double[][] positions = new double[][] {{0,-13.8},{2.5,-13.8},{0,-9.2},{2.5,-9.2},{0,-4.6},{2.5,-4.6},{0,0},{2.4,0}};


    private Collection<Beacon> beacons;
    private Map<Identifier, Double> availableBeacons = new ArrayMap<>();

    public CellIdentification(Collection<Beacon> beacons) {
        this.beacons = beacons;
    }

    public double[] getCurrentLocation(Collection<Beacon> beacons) {
        double[] pos = new double[2];
        if (availableBeacons != null && availableBeacons.size() != 0) {
            availableBeacons.clear(); // Refresh map by clearing
        }
        for (Beacon b : beacons) {
            Identifier instanceID = b.getId2();
            double distance = b.getDistance();
            availableBeacons.put(instanceID, distance);
        }
        Identifier check = findNearestBeacon(availableBeacons);
        int index = identifierToInt(check)-1;
        if (index > 0 && index < 8) {
            pos[0] = positions[index][0];
            pos[1] = positions[index][1];
        }
        return pos;
    }

    public Identifier findNearestBeacon(Map<Identifier, Double> beacons) {
        Double minimumDistance = 100.0;
        Identifier nearestBeacon = null;
        for (Map.Entry entry : beacons.entrySet()) {
            if ((Double) entry.getValue() < minimumDistance) {
                minimumDistance = (Double) entry.getValue();
                nearestBeacon = (Identifier) entry.getKey();
            }
        }
        return nearestBeacon;
    }

    public int identifierToInt(Identifier id) {
        int sum = 0;
        String str = id.toString();
        int index = 13;
        int p = 0;
        while (index > 1 && str.charAt(index) != '0'){
            int pow = (int) Math.pow(16.0,(double) p);
            if (str.charAt(index) > '0' && str.charAt(index) <= '9'){
                sum += (((int)str.charAt(index))-48)*pow;
            }else{
                sum += (((int)str.charAt(index))-87)*pow;
            }
            index--;
            p++;
        }
        return sum;
    }

}
